#!/usr/bin/env python3

from util import *
# NO OTHER IMPORTS ALLOWED 

def seam_carving(image, ncols):
    """
    Starting from the given image, use the seam carving technique to remove
    ncols (an integer) columns from the image.
    """
    raise NotImplementedError


# Optional Helper Functions

def cumulative_energy_map(energy):
    """
    Given a map of energy (e.g., the output of the compute_energy function),
    computes and returns a "cumulative energy map" as described.
    """
    raise NotImplementedError


def minimum_energy_seam(cem):
    """
    Given a cumulative energy map, returns a list of the indices into the
    'pixels' list that correspond to pixels contained in the minimum-energy
    seam.
    """
    raise NotImplementedError


def image_without_seam(im, s):
    """
    Given a color image and a list of indices to be removed from the image,
    return a new image (without modifying the original) that contains all the
    pixels from the original image, except those corresponding to the locations
    in the given list.
    """
    raise NotImplementedError


if __name__ == '__main__':
    # code in this block will only be run when you explicitly run your script,
    # and not when the tests are being run.  this is a good place for
    # generating images, etc.
    pass
